//
//	UWB_SimpleToon
//
//  Adapted liberally from David Wolff's CCSC-NW 2008 GLSL Workshop
//

float4x4 world;
float4x4 worldIT;
float4x4 view;
float4x4 viewI;
float4x4 projection;
float4x4 wvp;
float3 cameraPosition;
float offset;

// Light types
shared int POINTLIGHT = 0;
shared int DIRECTIONLIGHT = 1;
shared int SPOTLIGHT = 2;
        // shared variables can be shared across effects
        // e.g., another effect can use "extern" and get
        // access to this variable (need to use EffectPool parameter)
        

struct Light
{
	int type;				// 0=Point Light 1=Directional Light 2=Spotlight
	bool on;				// For switching light contribution on/off
	float4 ambient;
	float4 diffuse;
	float4 specular;
	float3 position;
	float3 direction;
	float range;
	float falloff;
	float3 attenuation;		// [0]=constant [1]=linear [2]=quadratic
	float theta;			// inner spotlight angle (umbra)
	float phi;				// outer spotlight angle (penumbra)
};

shared Light lights[4];
shared int numLights = 0;

struct Material
{
	float4 ambient;
	float4 diffuse;
	float4 specular;
	float4 emissive;
	float shininess;	// range 0 -> +infinity
};

shared Material material;

shared texture textureMap;
shared bool textureEnable;

sampler2D textureSampler = sampler_state
{
	Texture = <textureMap>;
    MagFilter = Linear;
    MinFilter = Anisotropic;
    MipFilter = Linear;
    MaxAnisotropy = 16;
};

sampler2D normalSampler = sampler_state 
{
    Texture = <textureMap>;
    MagFilter = Linear;
    MinFilter = Linear;
    MipFilter = Linear;
    AddressU = Wrap;
    AddressV = Wrap;
};

struct VertexShaderInput
{
	float3 Position : POSITION;
	float3 Normal : NORMAL;
	float2 TexCoord : TEXCOORD0;

};

struct VertexShaderOutput
{
     float4 Position : POSITION;
     float2 TexCoords : TEXCOORD0;
     float3 WorldNormal : TEXCOORD1;
     float3 WorldPosition : TEXCOORD2;
};

VertexShaderOutput LightingVS(VertexShaderInput input)
{
    VertexShaderOutput output;

    // world view projection matrix

     
	// transform the input position to the output
	output.Position = mul(float4(input.Position, 1.0), wvp);

	output.WorldNormal =  mul(input.Normal, worldIT);

	float4 worldPosition =  mul(float4(input.Position, 1.0), world);
	output.WorldPosition = worldPosition / worldPosition.w;
	
		
	
	output.TexCoords = input.TexCoord - float2(offset/5.0f, offset);
	//output.TexCoords.x = input.TexCoord.x + offset;
	//output.TexCoords.y = input.TexCoord.y + offset;
	if(output.TexCoords.x < 0)
		output.TexCoords.x = output.TexCoords.x + 511;
	if(output.TexCoords.y < 0)
		output.TexCoords.y = output.TexCoords.y + 511;

		
	

    return output;
}

float4 SingleLight(Light light, float3 worldPosition, float3 worldNormal, float4 texColor)
{
    float3 lightVector = light.position - worldPosition;
    float3 L = normalize(lightVector);
   
	float3 N = normalize(worldNormal);   // Surface normal
	float NdotL = dot(N, L);
	
	float scale = ceil(3.0 * NdotL) / 3.0; // scale is 0 or 1
	
	float4 color;
	if (textureEnable)
	    color = texColor;
	else
	    color = material.diffuse;
	
	color *= scale;
   
	return color;
}

float4 MultiLightPS(VertexShaderOutput input) : COLOR
{
	// Not using a global amient
	float4 color = material.diffuse;
	float4 texColor = float4(0.0f, 0.0f, 0.0f, 0.0f);
	float2 test2 = float2(40.0f,1.0f);
	texColor = tex2D(textureSampler, input.TexCoords);
	   
	color.a = texColor.r/(input.WorldPosition.z/5.0f);
	   
	float3 viewDirection = normalize(float3(viewI._41,viewI._42, viewI._43));
	//float3 viewDirection = float3(viewI._41,viewI._42, viewI._43);
	
	float3 N = normalize(input.WorldNormal);   // Surface normal 

    return color;
}

technique MultipleLights
{
    pass P0
    {
        VertexShader = compile vs_3_0 LightingVS();
        PixelShader = compile ps_3_0 MultiLightPS();
    }
}

